# © Copyright Databand.ai, an IBM Company 2022

import requests


def get_current_region():
    """
    works only from ec2 machine
    :return:
    """
    r = requests.get("http://169.254.169.254/latest/dynamic/instance-identity/document")
    response_json = r.json()
    return response_json.get("region")



# © Copyright Databand.ai, an IBM Company 2022

from dbnd import output, parameter
from dbnd._core.task.task import Task
from dbnd_aws.sagemaker.estimator_config import BaseEstimatorConfig
from dbnd_aws.sagemaker.sagemaker_ctrl import SageMakerCtrl
from targets import Target


class SageMakerTrainTask(Task):
    estimator_config = parameter[BaseEstimatorConfig]

    train = parameter(description="path to an s3 bucket where training data is stored")[
        Target
    ]
    test = parameter.none().help("path to an s3 bucket where test data is stored")[
        Target
    ]
    validate = parameter.none().help(
        "path to an s3 bucket where validation data is stored"
    )[Target]

    region = parameter(
        default="us-east-1", description="region to use for docker image resolution"
    )
    output_path = (
        output.description("s3 path to output a model")
        .folder_data.with_flag(None)
        .no_extension[Target]
    )
    sagemaker_role = parameter[str]

    wait_for_completion = parameter.c(default=True)[bool]
    print_log = parameter.c(default=True)[bool]
    check_interval = parameter.c(default=30)[int]
    max_ingestion_time = parameter.c(default=None)[int]

    def _get_ctrl(self):
        # type: ()-> SageMakerCtrl
        return SageMakerCtrl(self)

    def _task_submit(self):
        self._get_ctrl().train()
        return "Ok"



# © Copyright Databand.ai, an IBM Company 2022


class EmrClusterBuilder(object):
    def __init__(self, emr_client, ec2_client):
        super(EmrClusterBuilder, self).__init__()
        self.emr = emr_client
        self.ec2 = ec2_client  # boto3.client("ec2", region_name=region_name)

    def get_security_group_id(self, group_name, region_name):
        response = self.ec2.describe_security_groups(GroupNames=[group_name])
        return response["SecurityGroups"][0]["GroupId"]

    def create_cluster(
        self,
        region_name,
        cluster_name,
        release_label="emr-5.16.0",
        master_instance_type="m3.xlarge",
        num_core_nodes=2,
        core_node_instance_type="m3.2xlarge",
    ):
        emr_master_security_group_id = self.get_security_group_id(
            "ElasticMapReduce-master", region_name=region_name
        )
        emr_slave_security_group_id = self.get_security_group_id(
            "ElasticMapReduce-slave", region_name=region_name
        )
        cluster_response = self.emr.run_job_flow(
            Name=cluster_name,
            ReleaseLabel=release_label,
            Instances={
                "InstanceGroups": [
                    {
                        "Name": "Master nodes",
                        "Market": "ON_DEMAND",
                        "InstanceRole": "MASTER",
                        "InstanceType": master_instance_type,
                        "InstanceCount": 1,
                    },
                    {
                        "Name": "Slave nodes",
                        "Market": "ON_DEMAND",
                        "InstanceRole": "CORE",
                        "InstanceType": core_node_instance_type,
                        "InstanceCount": num_core_nodes,
                    },
                ],
                "KeepJobFlowAliveWhenNoSteps": True,
                "Ec2KeyName": "databand-dev",
                "EmrManagedMasterSecurityGroup": emr_master_security_group_id,
                "EmrManagedSlaveSecurityGroup": emr_slave_security_group_id,
            },
            VisibleToAllUsers=True,
            JobFlowRole="EMR_EC2_DefaultRole",
            ServiceRole="EMR_DefaultRole",
            Applications=[
                {"Name": "hadoop"},
                {"Name": "spark"},
                {"Name": "hive"},
                {"Name": "livy"},
                {"Name": "zeppelin"},
            ],
        )
        return cluster_response["JobFlowId"]
